#include <avr\io.h>
#include <avr\iom16.h>
#include <avr\interrupt.h>
#include <util\delay.h>
#include <stdbool.h>
#include "lcd.c"

#define F_CPU 1000000

#define sbi(x,y) x |= _BV(y) 			//set bit
#define cbi(x,y) x &= ~(_BV(y)) 		//clear bit

#define black true
#define white false

void InitPorts (void);					//initializes the ports
void InitInterrupts(void);
void InitPWM(void);						//initializes 8 bit PWM for the motors
void CalcError(void);
void LeftMF(void);						//left motor forward
void LeftMB(void);						//left motor backward
void LeftMS(void);						//left motor stop
void RightMB(void);						//right motor backward
void RightMS(void);						//right motor stop
void RightMF(void);						//left motor forward
void delay(float);
void BacklightOn(void);
void BacklightOff(void);
void DispAdjusmentMenu(void);
void DispMenu(int);

bool pause = true;
char lastreading = 'r';
unsigned int leftpulse, rightpulse,s1, s2, s3, s4, s5, s6, s7, s8, maxspeed = 250;
float error = 0, perror = 0;
unsigned short int menu = 0;
unsigned short int basespeed = 50, Kp = 25 , Ki = 1,Kd = 2;
float P, I, D, correction;

void InitPorts(void) 						//initialize the pins 
{
	sbi(DDRC, PC7);						//set LCD Baklight pin as output
	cbi(PORTC, PC7);		
	
	//initializes the pins connected to motor driver as output
	sbi(DDRD, PD5);						
	sbi(DDRD, PD6);
	sbi(DDRD, PD7);	
	sbi(DDRD, PD4);
	sbi(DDRD, PD0);
	sbi(DDRD, PD1);
	cbi(PORTD, PD5);
	cbi(PORTD, PD6);
	cbi(PORTD, PD7);	
	cbi(PORTD, PD4);
	cbi(PORTD, PD0);
	cbi(PORTD, PD1);	
	
	//initiallizes the motor enable pins as output
	sbi(PORTD, PD4);	
	sbi(PORTD, PD5);		
	

	
	//the following statements set the seitch pins as input pins
	cbi(DDRD, PD2);
	cbi(DDRD, PD3);
	cbi(DDRB, PB2);
	
	DDRA = 0;    								//sets the port connected to sensors as input	
	PORTA = 0;   								//enable pull up	
}
	
void InitPWM(void)								//initialize PWM				
{	
	TCCR1A = _BV(WGM10)   					    // 8 bit fast PWM
		   | _BV(COM1A1)  						// set OC1A/B on compare match
		   | _BV(COM1B1);		
	TCCR1B = _BV(CS11)  	   				   // 1/8 prescale fast PWM
		   | _BV(WGM12) ;  				
}	

void InitInterrupts(void)
{
	cbi(MCUCR, ISC10);
	cbi(MCUCR, ISC11);
	
	cbi(MCUCR, ISC00);
	cbi(MCUCR, ISC01);
	
	cbi(MCUCR, ISC2);
	
	GICR  |= (1<<INT0);
	GICR  |= (1<<INT1);
	GICR  |= (1<<INT2);
}

void CalcError()								//calculates the current error
{				
	s1 = 0;
	s2 = 0;
	s3 = 0;
	s4 = 0;
	s5 = 0;
	s6 = 0;	
	s7 = 0; 
	s8 = 0;
	
	if(bit_is_clear(PINA, PA7))		
	{
		lastreading = 'l';
		s1 = 1;	
	}
	if(bit_is_clear(PINA, PA6))	
		s2 = 1;
	if(bit_is_clear(PINA, PA5))	
		s3 = 1;	
	if(bit_is_clear(PINA, PA4))		
		s4 = 1;		
	if(bit_is_clear(PINA, PA3))
		s5 = 1;		
	if(bit_is_clear(PINA, PA2))		
		s6 = 1;				
	if(bit_is_clear(PINA, PA1))		
		s7 = 1;	
	if(bit_is_clear(PINA, PA0))
	{
		lastreading = 'r';
		s8 = 1;	
	}		
	
	perror = error;
	
	//the following statements calculate the error
	
	error = (s1 * 1) + (s2 * 2) + (s3 * 3) + (s4 * 4) + (s5 * 5) + (s6 * 6) +(s7 * 7) + (s8*8);
	error = (error)/(s1+s2+s3+s4+s5+s6+s7+s8);	
	error = error - 4.5;
}

void LeftMB(void)	
{
	sbi(PORTD, PD5);
	cbi(PORTD, PD6);
	sbi(PORTD, PD7);
}

void RightMF(void)
{
	sbi(PORTD, PD4);
	cbi(PORTD, PD0);
	sbi(PORTD, PD1);
}
void LeftMF(void)
{
	sbi(PORTD, PD5);
	sbi(PORTD, PD6);
	cbi(PORTD, PD7);
}

void RightMB(void)
{
	sbi(PORTD, PD4);
	sbi(PORTD, PD0);
	cbi(PORTD, PD1);
}
void LeftMS(void)
{
	sbi(PORTD, PD5);
	sbi(PORTD, PD6);
	sbi(PORTD, PD7);
}

void RightMS(void)
{
	sbi(PORTD, PD4);
	sbi(PORTD, PD0);
	sbi(PORTD, PD1);
}

void delay(float n)
{
	n *= 10;
	for(int i = 0; i < n; i++)
		_delay_ms(F_CPU/10000);
}

void BacklightOn(void)
{
	sbi(PORTC, PC7);
}

void BacklightOff(void)
{
	cbi(PORTC, PC7);
}